/*
 * Copyright (c) 2014 Faust Edition development team.
 *
 * This file is part of the Faust Edition.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

YUI.add('text-display', function (Y) {

	/**
	 * An area in which text is displayed.
	 * 
	 * All arguments are passed as properties of a configuration object.
	 * 
	 * @param container reference to HTML node acting as output container
	 * @param text instance of Y.Faust.Text which shall be displayed
	 * @param cssPrefix string acting as prefix to all CSS-names generated by this class (optional)
	 * @param renderCallback callback function for handling special rendering (optional).
	 *                       This function is called for every processed annotation and receives the following arguments:
	 *                       	annotation      instance of Y.Faust.Annotation
	 *                       	cssPrefix       the css Prefix configured above
	 *                       	partitionNode   the DOM node of the currently rendered text partition
	 *                       	lineNode        the DOM node of the currently rendered line of text
	 *                       	isFirst         boolean indicating whether this partition constitutes the beginning of the annotation
	 *                       	isLast          boolean indicating whether this partition constitutes the end of the annotation
	 *                       It shall return an array of strings which will be added as CSS classes to the current partition.
	 * 
	 */
	var TextDisplayView = Y.Base.create('textDisplayView', Y.View, [], {
		/**
		 * Outputs an HTML representation of the text to the container
		 * @param range Y.Faust.Range Text range to render
		 */
		render : function(range) {
			var text = this.get('text');
			var container = this.get('container');
			var prefix = this.get('cssPrefix');
			var callback = this.get('renderCallback');
			
			range = range || new Y.Faust.Range(0, text.contentLength);

			var partitions = text.partition(null, range.start, range.end);
			
			Y.Array.each(partitions, function(partition, i, partitions){
				var partitionNode = Y.Node.create('<span></span>'); //quite expensive
				container.append(partitionNode);
				var lineNumNode = null;
				
				var annotations = text.find(partition.start, partition.end);
				
				Y.Array.each(partition.of(text.content).split('\n'), function(line, n) {
					if(n > 0)
						partitionNode.append('<br>');
					
					var lineNode = Y.config.doc.createTextNode(line);
					partitionNode.append(lineNode);
					
					var classes = [];
					Y.Array.each(annotations, function(annotation){

						var name = prefix + annotation.name.localName;
						
						classes.push(name);
						classes.push(prefix + annotation.id);
						
						var first = (i == 0 || annotation.targetIn(text).range.start == partition.start);
						var last = (i+1 == partitions.length || annotation.targetIn(text).range.end == partition.end);
						
						if(first)
							classes.push(name + '-first');
						if(last)
							classes.push(name + '-last');
						
						classes = classes.concat(callback(annotation, prefix, partitionNode, lineNode, first, last));
					});
					partitionNode.addClass(classes.join(' '));
				}, this);
			}, this);
		}
    }, {
		ATTRS: {
			container: {}, 
			text: {},
			cssPrefix: {value: ""},
			renderCallback: {value: function(){}}
		}
	});
	
	Y.mix(Y.namespace("Faust"), {
		TextDisplayView: TextDisplayView
	});
}, '0.0', {
	requires: ['base', 'view']
});